import 'package:flutter/material.dart';

enum BuildType{
  DEBUG,
  PROFILE,
  RELEASE,
}

class BuildConfig<V> {
  final BuildType buildType = appBuildType;
  final String packageName;
  final String appName;
  final String versionCode;
  final String versionName;
  final V variant;

  BuildConfig({
    @required this.packageName,
    @required this.appName,
    @required this.versionCode,
    @required this.versionName,
    this.variant,
  });

  static BuildType get appBuildType {
    if (const bool.fromEnvironment('dart.vm.product'))
      return BuildType.RELEASE;
    var result = BuildType.PROFILE;
    //Little trick, since assert only runs on DEBUG mode
    assert(() {
      result = BuildType.DEBUG;
      return true;
    }());
    return result;
  }

  bool get isDebug => buildType == BuildType.DEBUG;
  bool get isProfile => buildType == BuildType.PROFILE;
  bool get isRelease => buildType == BuildType.RELEASE;

  @override
  String toString() =>
      "buildType: $buildType,\n"
      "packageName: $packageName,\n"
      "appName: $appName,\n"
      "versionCode: $versionCode,\n"
      "versionName: $versionName,\n"
      "variant: $variant,\n"
  ;
}