import 'package:adeptutils/src/extendables/jsonable.dart';
import 'package:adeptutils/src/collections/multi_entry_set.dart';
import 'package:adeptutils/src/collections/jmap_entry.dart';

class MultiKeyMapUtils{
  static Map<String, dynamic> toJson(MultiKeyMap<String, dynamic> value) =>
      value != null ? value.toJson() : null;

  static MultiKeyMap<String, dynamic> fromJson(Map<String, dynamic> value) =>
      MultiKeyMap<String, dynamic>.fromJson(value);
}

class MultiKeyMap<K, V> extends Jsonable<MultiKeyMap> implements Map<K, V>
{
  List<JMapEntry<K, V>> _items;

  MultiKeyMap() {
    _items = [];
  }

  List<JMapEntry<K, V>> get items => _items;

  set items(List<JMapEntry<K, V>> value) => _items = value ?? _items;

  @override
  bool get isNotEmpty => _items.isNotEmpty;

  @override
  bool get isEmpty => _items.isEmpty;

  @override
  int get length => _items.length;

  @override
  Iterable<V> get values {
    List<V> values = [];
    for(JMapEntry<K, V> entry in _items) values.add(entry.value);
    return values;
  }

  @override
  Iterable<K> get keys {
    Set<K> keySet = new MultiEntrySet();
    for(JMapEntry<K, V> entry in _items) keySet.add(entry.key);
    return keySet;
  }

  @override
  void forEach(void f(K key, V value)) {
    _items.forEach((entry){f(entry.key, entry.value);});
  }

  @override
  void clear() {
    _items = [];
  }

  @override
  V remove(Object key) {
    int i = 0;
    for(JMapEntry<K, V> entry in _items){
      if(entry.key == key) {
        V value = entry.value;
        _items.remove(i);
        return value;
      }
      ++i;
    }
    return null;
  }

  @override
  void addAll(Map<K, V> other) {
    if(other == null) return;
    for(MapEntry entry in other.entries)
      _items.add(JMapEntry(entry.key, entry.value));
  }

  @override
  V putIfAbsent(K key, V ifAbsent()) {
    _items.forEach((entry){
      print(entry);
    });
    return null;
  }

  @override
  void removeWhere(bool predicate(K key, V value)) {
    _items.forEach((entry){
      print(entry);
    });
  }

  @override
  void updateAll(V update(K key, V value)) {
    _items.forEach((entry){
      print(entry);
    });
  }

  @override
  V update(K key, V update(V value), {V ifAbsent()}) {
    _items.forEach((entry){
      print(entry);
    });
    return null;
  }

  @override
  void addEntries(Iterable<MapEntry<K, V>> newEntries) {
    if(newEntries == null) return;
    newEntries.forEach((entry){
      _items.add(JMapEntry(entry.key, entry.value));
    });
  }

  @override
  Iterable<MapEntry<K, V>> get entries {
    List<MapEntry<K,V>> list = [];
    _items.forEach((entry) => list.add(MapEntry(entry.key, entry.value)));
    return list;
  }

  @override
  void operator []=(K key, V value) => _items.add(JMapEntry<K, V>(key, value));

  @override
  V operator [](Object key) {
    for(JMapEntry<K, V> entry in _items) if(entry.key == key) return entry.value;
    return null;
  }

  @override
  bool containsKey(Object key) {
    for(JMapEntry entry in _items) if(entry.key == key) return true;
    return false;
  }

  @override
  bool containsValue(Object value) {
    for(JMapEntry entry in _items) if(entry.value == value) return true;
    return false;
  }

  @override
  Map<K2, V2> map<K2, V2>(MapEntry<K2, V2> f(K key, V value)) {
    return {};
  }

  @override
  Map<RK, RV> cast<RK, RV>() {
    return {};
  }

  List<JMapEntry<K, V>> toList() => _items;

  @override
  Map<String, dynamic> toJson() => <String, dynamic>{'multi_key_map_items': _items};

  @override
  MultiKeyMap<String, dynamic> fromJsonMap(Map<String, dynamic> json) => MultiKeyMap.fromJson(json);

  factory MultiKeyMap.fromJson(Map<String, dynamic> json) {
    if(json == null) return null;
    if(json.containsKey('multi_key_map_items'))
      return MultiKeyMap<K, V>()..items = (
          (json['multi_key_map_items'] as List)
              ?.map((e) =>
          e == null ? null : JMapEntry<K, V>.fromJson(e as Map<String, dynamic>))
              ?.toList()
      );
    List<JMapEntry<K, V>> items = [];
    json.forEach((key, value) {
      if(key != null)
        items.add(JMapEntry<K, V>.fromJson({'key': key, 'value': value}));
    });
    return MultiKeyMap()..items = items;
  }

}
