
import 'dart:async';

enum LifecycleState{
  created,
  resumed,
  inactive,
  paused,
  detached,
  stopped,
  destroyed,
}

class StreamSubscriptionLifecycleAware {
  final StreamSubscription streamSubscription;
  final Set<LifecycleState> lifecycleStates;

  StreamSubscriptionLifecycleAware({this.streamSubscription,
      this.lifecycleStates});

  void onLifecycleChanged(LifecycleState lifecycleState){
    switch(lifecycleState){
      case LifecycleState.resumed:
        resume(LifecycleState.resumed);
        break;
      case LifecycleState.paused:
        pause(LifecycleState.paused);
        break;
      case LifecycleState.stopped:
        pause(LifecycleState.stopped);
        break;
      case LifecycleState.destroyed:
        cancel(LifecycleState.destroyed);
        break;
      default:
    }
  }

  bool allowed(LifecycleState lifecycleState) => lifecycleStates == null || lifecycleStates.contains(lifecycleState);

  resume(LifecycleState lifecycleState) {
    if(allowed(lifecycleState)) streamSubscription?.resume();
  }

  pause(LifecycleState lifecycleState, [Future resumeSignal]) {
    if(allowed(lifecycleState)) streamSubscription?.pause(resumeSignal);
  }

  cancel(LifecycleState lifecycleState) {
    if(allowed(lifecycleState)) streamSubscription?.cancel();
  }

}