import 'dart:convert';

import 'package:adeptutils/adeptutils.dart';

abstract class Jsonable<T extends Object>{

  static const int jsonParserError = 1000;

  ///
  /// This is just to suggest children to implement a named constructor to
  /// support creating objects fromJson(...) and fromJsonString(...) as easy as
  /// T.build().fromJson(...) or T.build().fromJsonString(...)

  @override
  String toString() => toJsonString();

  Map<String, dynamic> toJson();
  Map<String, dynamic> toJsonMap() => toJson();
  T fromJsonMap(Map<String, dynamic> json);

  String toJsonString() => jsonEncode(this);
  T fromJsonString(String json) => TextUtils.isNotEmpty(json) ? fromJsonMap(jsonDecode(json)) : null;
  List<T> fromJsonStringList(String jsonStringList) => TextUtils.isNotEmpty(jsonStringList) ? fromJsonList(jsonDecode(jsonStringList)) : null;

  List<T> fromJsonList(List<dynamic> jsonList){
    if(jsonList == null) return null;
    if(jsonList.isEmpty) return [];
    List<T> list = jsonList.map((dynamic json) {
      if(json is! Map<String, dynamic>)
        throw new Exception("fromJsonList(List<dynamic> jsonList) expects a Json Map<String, dynamic> but other object was found instead");
      return fromJsonMap(json as Map<String, dynamic>);
    }).toList();
    return list;
  }

///  Factory constructor is required to work with automatic json serialization
///  factory T.fromJson(Map<String, dynamic> json) => _$TFromJson(json);
}