import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'dart:async';

class TextUtils{

  static bool isEmpty(String text) => text == null || text.isEmpty || text.trim().isEmpty;
  static bool isNotEmpty(String text) => !isEmpty(text);
  static bool equals(String a, String b) => a == b;
}

class AdeptTextEditingController extends TextEditingController {
  String _error;

  AdeptTextEditingController({String text}) : super (text: text);

  bool get hasError => error != null;
  bool get isEmpty => text == null || text.isEmpty;
  bool get isNotEmpty => !isEmpty;
  String get error => _error;

  set error(String value) {
    _error = value;
    if(_error != null) _feedback();
  }

  void setError(String error, {bool notify = true}) {
    this.error = error;
    if(notify) notifyListeners();
  }

  void _feedback() async{
    for(int i = 0; i < 3; ++i)
      await Future.delayed(Duration(milliseconds: 100), () => HapticFeedback.vibrate());
  }

  void clearError({bool notify = true}) {
    _error = null;
    if(notify) notifyListeners();
  }

  @override
  void clear() {
    clearError();
    super.clear();
  }

}
class AdeptValueController<T> extends ValueNotifier<T> {
  String _error;

  AdeptValueController({T value}) : super (value);

  bool get hasError => error != null;

  bool get isEmpty => value == null;

  String get error => _error;

  set error(String value) {
    _error = value;
    if(_error != null) _feedback();
  }

  void setError(String error, {bool notify = true}) {
    this.error = error;
    if(notify) notifyListeners();
  }

  void _feedback() async{
    for(int i = 0; i < 3; ++i)
      await Future.delayed(Duration(milliseconds: 100), () => HapticFeedback.vibrate());
  }

  void clearError({bool notify = false}) {
    _error = null;
    if(notify) notifyListeners();
  }

  void clear() {
    clearError();
    value = null;
  }

}